<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Wp_Protect_Password
 * @subpackage Wp_Protect_Password/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wp_Protect_Password
 * @subpackage Wp_Protect_Password/includes
 * @author     BWPS <hello@ymese.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if ( ! class_exists( 'Wp_Protect_Password' ) ) {

	class Wp_Protect_Password {

		/**
		 * The loader that's responsible for maintaining and registering all hooks that power
		 * the plugin.
		 *
		 * @since    1.0.0
		 * @access   protected
		 * @var      Wp_Protect_Password_Loader $loader Maintains and registers all hooks for the plugin.
		 */
		protected $loader;

		/**
		 * The unique identifier of this plugin.
		 *
		 * @since    1.0.0
		 * @access   protected
		 * @var      string $plugin_name The string used to uniquely identify this plugin.
		 */
		protected $plugin_name;

		/**
		 * The current version of the plugin.
		 *
		 * @since    1.0.0
		 * @access   protected
		 * @var      string $version The current version of the plugin.
		 */
		protected $version;

		/**
		 * Define the core functionality of the plugin.
		 *
		 * Set the plugin name and the plugin version that can be used throughout the plugin.
		 * Load the dependencies, define the locale, and set the hooks for the admin area and
		 * the public-facing side of the site.
		 *
		 * @since    1.0.0
		 */
		public function __construct() {
			if ( defined( 'PDA_NAME_VERSION' ) ) {
				$this->version = PDA_NAME_VERSION;
			} else {
				$this->version = '1.0.0';
			}
			$this->plugin_name = 'wp-protect-password';

			$this->load_dependencies();
			$this->set_locale();
			$this->define_admin_hooks();
			$this->define_public_hooks();
		}

		/**
		 * Load the required dependencies for this plugin.
		 *
		 * Include the following files that make up the plugin:
		 *
		 * - Wp_Protect_Password_Loader. Orchestrates the hooks of the plugin.
		 * - Wp_Protect_Password_i18n. Defines internationalization functionality.
		 * - Wp_Protect_Password_Admin. Defines all hooks for the admin area.
		 * - Wp_Protect_Password_Public. Defines all hooks for the public side of the site.
		 *
		 * Create an instance of the loader which will be used to register the hooks
		 * with WordPress.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function load_dependencies() {

			/**
			 * The class responsible for orchestrating the actions and filters of the
			 * core plugin.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-protect-password-loader.php';

			/**
			 * The class responsible for defining internationalization functionality
			 * of the plugin.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-protect-password-i18n.php';

			/**
			 * Helper functions
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . '/includes/class-wp-protect-password-functions.php';

			/**
			 * Update service functions
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-protect-password-update-service.php';

			/**
			 * Shared service between gold and free
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-protect-password-service.php';

			/**
			 * The class responsible for defining all actions that occur in the public-facing
			 * side of the site.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wp-protect-password-public.php';

			/**
			 * This class responsible for plugin's constants
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . '/includes/class-wp-protect-password-constants.php';

			/**
			 * Plugin update checker
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-plugin-update-checker/plugin-update-checker.php';

			if ( wpp_is_gold_function() ) {
				/**
				 * Ymese plugin's sdk
				 */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

				/**
				 * This class responsible for installing table.
				 */
				require_once PDA_PWD_DIR_PATH . '/includes/gold/class-wp-protect-password-db.php';

				/**
				 * This class responsible for helper functions.
				 */
				require_once PDA_PWD_DIR_PATH . '/includes/gold/class-wp-protect-password-functions-gold.php';

				/**
				 * Service class
				 */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . '/includes/gold/class-wp-protect-password-service.php';

				/**
				 * Repository
				 */
				require_once PDA_PWD_DIR_PATH . '/includes/gold/class-wp-protect-password-repo.php';

				if ( wpp_is_entered_license() ) {
					/**
					 * API
					 */
					require_once PDA_PWD_DIR_PATH . '/includes/gold/class-wp-protect-password-api.php';
				}
			}

			/**
			 * The class responsible for defining all actions that occur in the admin area.
			 */
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wp-protect-password-admin.php';

			if ( wpp_is_gold_function() ) {
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/gold/class-wp-protect-password-admin-gold.php';
			}

			$this->loader = new Wp_Protect_Password_Loader();
		}

		/**
		 * Define the locale for this plugin for internationalization.
		 *
		 * Uses the Wp_Protect_Password_i18n class in order to set the domain and to register the hook
		 * with WordPress.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function set_locale() {

			$plugin_i18n = new Wp_Protect_Password_i18n();

			$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
		}

		/**
		 * Register all of the hooks related to the admin area functionality
		 * of the plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function define_admin_hooks() {

			$plugin_admin = new Wp_Protect_Password_Admin( $this->get_plugin_name(), $this->get_version() );
			$this->loader->add_action( 'admin_menu', $plugin_admin, 'wp_protect_password_create_plugin_menu' , 10);
			$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
			$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
			// Protect Password.
			$this->loader->add_action( 'login_form_ppwp_password_action', $plugin_admin, 'pup_post_pass' );
			$this->loader->add_action( 'wp_ajax_wp_protect_password_update_password', $plugin_admin, 'wp_protect_password_update_password' );
			$this->loader->add_action( 'wp_ajax_wp_protect_password_save_password', $plugin_admin, 'wp_protect_password_save_password' );
			$this->loader->add_action( 'wp_ajax_wp_protect_password_update_settings', $plugin_admin, 'wp_protect_password_update_settings' );
			$this->loader->add_action( 'template_redirect', $plugin_admin, 'render_form_enter_password' );

			// WordPress function filter
			$this->loader->add_filter( 'post_password_required', $plugin_admin, 'wp_protect_password_ppr', 10, 2 );

			//Multiple password
			if ( ! wpp_is_gold_function() ) {
				$this->loader->add_action( 'admin_init', $plugin_admin, 'add_custom_metabox_to_edit_page_for_free_version' );
			}

			if ( wpp_is_gold_function() ) {
				$plugin_admin_gold = new Wp_Protect_Password_Admin_Gold( $this->get_plugin_name(), $this->get_version() );
				if ( wpp_is_entered_license() ) {
					$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin_gold, 'gold_enqueue_styles' );
					$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin_gold, 'gold_enqueue_scripts' );

					$this->loader->add_filter( 'manage_posts_columns', $plugin_admin_gold, 'wp_protect_password_post_custom_column' );
					$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin_gold, 'wp_protect_password_post_custom_column_content', 10, 2 );

					$this->loader->add_filter( 'manage_pages_columns', $plugin_admin_gold, 'wp_protect_password_page_custom_column' );
					$this->loader->add_action( 'manage_pages_custom_column', $plugin_admin_gold, 'wp_protect_password_page_custom_column_content', 10, 2 );

					$this->loader->add_action( 'rest_api_init', $plugin_admin_gold, 'wppp_rest_api_init_cb', 10, 2 );

					$this->loader->add_filter( 'manage_edit-category_columns', $plugin_admin_gold, 'add_column_to_category_table', 10 );
					$this->loader->add_filter( 'manage_category_custom_column', $plugin_admin_gold, 'render_data_for_category_table', 10, 3 );

					$this->loader->add_action( 'wp_ajax_wp_protect_password_update_category_protect', $plugin_admin_gold, 'wp_protect_password_update_category_protect' );

					$this->loader->add_action( 'upgrader_process_complete', $plugin_admin_gold, 'handle_plugin_update', 9 );

					$this->loader->add_action( 'upgrader_process_complete', $plugin_admin_gold, 'handle_update_default_setting', 10 );

					$this->loader->add_action( 'admin_init', $plugin_admin_gold, 'handle_plugin_update', 9 );

					$this->loader->add_action( 'woocommerce_before_single_product', $plugin_admin_gold, 'ppwp_handle_before_view_product_woo' );

					$this->loader->add_action( 'admin_init', $plugin_admin_gold, 'add_custom_metabox_to_edit_page' );
					$this->loader->add_filter( 'ppwp_customize_password_form_message', $plugin_admin_gold, 'customize_password_form_message' );


				}
				// Notification.
				$this->loader->add_action( 'admin_notices', $plugin_admin_gold, 'wp_protect_password_notices' );
			}

			$this->loader->add_action( 'wp_ajax_wp_protect_password_gold_check_license', $plugin_admin, 'wp_protect_password_gold_check_license' );

			$this->loader->add_action( 'admin_init', $plugin_admin, 'handle_update_for_plugin' );

			$this->loader->add_action( 'wp_head', $plugin_admin, 'wp_protect_password_custom_header' );
		}

		/**
		 * Register all of the hooks related to the public-facing functionality
		 * of the plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 */
		private function define_public_hooks() {

			$plugin_public = new Wp_Protect_Password_Public( $this->get_plugin_name(), $this->get_version() );

			$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
			$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
			$this->loader->add_filter( 'the_content', $plugin_public, 'pup_filter_content', 99999999 );
		}

		/**
		 * Run the loader to execute all of the hooks with WordPress.
		 *
		 * @since    1.0.0
		 */
		public function run() {
			$this->loader->run();
		}

		/**
		 * The name of the plugin used to uniquely identify it within the context of
		 * WordPress and to define internationalization functionality.
		 *
		 * @return    string    The name of the plugin.
		 * @since     1.0.0
		 */
		public function get_plugin_name() {
			return $this->plugin_name;
		}

		/**
		 * The reference to the class that orchestrates the hooks with the plugin.
		 *
		 * @return    Wp_Protect_Password_Loader    Orchestrates the hooks of the plugin.
		 * @since     1.0.0
		 */
		public function get_loader() {
			return $this->loader;
		}

		/**
		 * Retrieve the version number of the plugin.
		 *
		 * @return    string    The version number of the plugin.
		 * @since     1.0.0
		 */
		public function get_version() {
			return $this->version;
		}

	}
}
