<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Wp_Protect_Password
 * @subpackage Wp_Protect_Password/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Wp_Protect_Password
 * @subpackage Wp_Protect_Password/public
 * @author     BWPS <hello@ymese.com>
 */
class Wp_Protect_Password_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Protect_Password_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Protect_Password_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wp-protect-password-public.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Protect_Password_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Protect_Password_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
	}

	/**
	 * Handle before send content to client
	 *
	 * @param array $content Data from client.
	 *
	 * @return bool|string|string[]|null
	 */
	public function pup_filter_content( $content ) {
		$post = get_post();
		if ( is_null( $post ) ) {
			return $content;
		}
		// Handle gold function.
		if ( wpp_is_gold_function() ) {
			// Return content when role access.
			if ( $this->wpp_handle_whitelist_roles( $post->ID ) ) {
				return $content;
			}
			if ( is_protected_page_post( $post->ID ) ) {
				return $this->gold_content_filter( $post, $content );
			} else {
				return $content;
			}
		} else {
			return $this->free_content_filter( $post, $content );
		}
	}

	/**
	 * Filter content for gold version
	 *
	 * @param array $post Data from client.
	 * @param string $content Data from client.
	 *
	 * @return mixed|string|string[]|null
	 */
	private function gold_content_filter( $post, $content ) {
		$ppwp_auto_protect_all_child_pages = wpp_get_settings_boolean( WP_Protect_Password_Constant::WPP_AUTO_PROTECT_ALL_CHILD_PAGES );
		if ( $ppwp_auto_protect_all_child_pages ) {
			$parents   = get_post_ancestors( $post->ID );
			$parent_id = $parents ? $parents[ count( $parents ) - 1 ] : $post->ID;
		} else {
			$parent_id = $post->ID;
		}

		$service = new WP_Protect_Password_Service();
		$is_ok   = $service->is_valid_cookie( $parent_id );

		if ( $is_ok ) {
			return $service->replace_private_link_type_password_for_content( $content );
		}

		return $this->render_login_form( $post );
	}

	/**
	 * Filter content for free version
	 *
	 * @param array $post Data from client.
	 * @param string $content Data from client.
	 *
	 * @return bool|string
	 */
	private function free_content_filter( $post, $content ) {
		$post_id = $post->ID;

		// 1. Get passwords.
		$service   = new WP_Protect_Password_Shared_Service();
		$result    = $service->get_passwords( $post_id );
		$passwords = $result['passwords'];
		if ( ! $result['has_global_passwords'] && ! $result['has_role_passwords'] ) {
			return $content;
		}

		// 2. Check password.
		if ( $service->is_valid_cookie( $post_id, $passwords ) ) {
			return $content;
		}

		// 3. Form rendering.
		if ( $result['has_global_passwords'] ) {
			return $this->render_login_form( $post );
		}

		if ( $result['has_role_passwords'] && $result['has_current_role_password'] ) {
			return $this->render_login_form( $post );
		}

		return '<p><strong>This page is protected. Please try again or contact the website owner.</strong></p>';
	}

	/**
	 * Render form login
	 *
	 * @param array $post Data from client.
	 *
	 * @return mixed
	 */
	public function render_login_form( $post ) {
		$current_url = ( isset( $_SERVER['HTTPS'] ) && 'on' === $_SERVER['HTTPS'] ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
		$query_str   = parse_url( $current_url, PHP_URL_QUERY );
		parse_str( $query_str, $query_params );

		if ( wpp_is_gold_function() ) {
			$error_message = wpp_get_settings_value( WP_Protect_Password_Constant::WPP_ERROR_MESSAGE );
			$allowed_html  = wp_kses_allowed_html( 'post' );
			$error_message = wp_kses( $error_message, $allowed_html );
		} else {
			$error_message = '';
		}

		$enter_wrong_password = '';
		if ( array_key_exists( 'ppwp_enter_wrong_password', $query_params ) && 'true' === $query_params['ppwp_enter_wrong_password'] ) {
			$enter_wrong_password = apply_filters( 'ppwp_text_for_entering_wrong_password', '<p class="ppwp-wrong-pw-error">' . ( empty( $error_message ) ? 'Please enter the correct password!' : $error_message ) . '</p>' );
		}

		$label = 'pwbox-' . ( empty( $post->ID ) ? rand() : $post->ID );

		$form_message = apply_filters( 'ppwp_customize_password_form_message', WP_Protect_Password_Constant::WPP_DEFAULT_FORM_MESSAGE );

		$default_element = '<p>' . __( $form_message ) . '</p>'
		                   . $enter_wrong_password . '<p><label for="' . $label . '">' . __( 'Password:' ) . ' <input name="post_password" id="' . $label . '" type="password" size="20" /></label>'
		                   . '<input type="submit" name="Submit" value="' . esc_attr_x( 'Enter', 'post password form' ) . '" /></p>';

		$content_form = apply_filters( 'ppwp_customize_password_form', $default_element, $post->ID, $enter_wrong_password );

		$output = '<form action="' . esc_url( site_url( 'wp-login.php?action=ppwp_password_action', 'login_post' ) ) . '" class="post-password-form" method="post">'
		          . $content_form
		          . '<input type="hidden" name="post_id" value="' . $post->ID . '" />'
		          . '</form>';

		return apply_filters( 'the_password_form', $output );
	}

	/**
	 * Handle user access
	 *
	 * @param string $post_id Post id.
	 *
	 * @return bool
	 */
	public function wpp_handle_whitelist_roles( $post_id ) {
		$service = new WP_Protect_Password_Service();

		return $service->is_whitelist_role( $post_id );
	}


}
